import chimera, numpy
from numpy import *
from StructBio.Scenographics.solids import Solids
from StructBio.SSEs.axesForSSEs import AxesForSSEs


#==============================================================================
# TubesForSSEs CLASS
#==============================================================================
# This class generates the tube solids that give a visual representation of the
# primary axis going through the designated seconday structural elements (helices
# or strands or both).  The axes are generated by an instantiation of the
# axesForSSEs class  which processes the molecular structure to extract the various
# secondary structures.  An axesForSSEs object uses the inertialAxis class to
# generate each primary axis).
class TubesForSSEs(object):
    def __init__(self, protMol, tolerance = 3.5, minLength = 3.0, chainId = "",
                 helices = True, helixTubeRadius = 1.5, helixTubeColor = (1.000, 0.843, 0.000),
                 strands = True, strandTubeRadius = 0.8, strandTubeColor = (0.000, 0.843, 1.000)):
        self.prot = protMol
        self.tolerance = tolerance
        self.minLength = minLength
        self.chainId = chainId
        self.helices = helices
        self.helixTubeRadius = helixTubeRadius
        self.helixTubeColor = helixTubeColor
        self.strands = strands
        self.strandTubeRadius = strandTubeRadius
        self.strandTubeColor = strandTubeColor        
        
        self.axes = AxesForSSEs(self.prot, self.tolerance, self.minLength, self.chainId, self.helices, self.strands)

        # Generate tube display:
        self.__tubeDisp = Solids()
        for axes in self.axes.inertialAxes_L:
            if axes.atoms_L[0].residue.isHelix:
                self.__tubeDisp.addTube(axes.startPoint, axes.endPoint, self.helixTubeRadius, self.helixTubeColor)
            if axes.atoms_L[0].residue.isStrand:
                self.__tubeDisp.addTube(axes.startPoint, axes.endPoint, self.strandTubeRadius, self.strandTubeColor)
        self.__tubeDisp.display()
        
    def show(self):
        self.__tubeDisp.show()

    def hide(self):
        self.__tubeDisp.hide()
